@@ -57,7 +57,9 @@
 **     self    The bucket
 **     keyarg    The key to look for
 **     has_key    Boolean; if true, return a true/false result; else return
-**              the value associated with the key.
+**              the value associated with the key. When true, ignore the TypeError from
+**              a key conversion issue, instead
+**              transforming it into a KeyError.
 **
 ** Return
 **     If has_key:
@@ -81,7 +83,15 @@ _bucket_get(Bucket *self, PyObject *keyarg, int has_key)
     int copied = 1;
 
     COPY_KEY_FROM_ARG(key, keyarg, copied);
-    UNLESS (copied) return NULL;
+    UNLESS (copied)
+    {
+        if (has_key && PyErr_ExceptionMatches(PyExc_TypeError))
+        {
+            PyErr_Clear();
+            PyErr_SetObject(PyExc_KeyError, keyarg);
+        }
+        return NULL;
+    }
 
     UNLESS (PER_USE(self)) return NULL;
 
@@ -106,7 +116,17 @@ Done:
 static PyObject *
 bucket_getitem(Bucket *self, PyObject *key)
 {
-    return _bucket_get(self, key, 0);
+    PyObject* result;
+
+    result = _bucket_get(self, key, 0);
+
+    if (result == NULL && PyErr_ExceptionMatches(PyExc_TypeError))
+    {
+        PyErr_Clear();
+        PyErr_SetObject(PyExc_KeyError, key);
+    }
+
+    return result;
 }
 
 /*
@@ -1347,11 +1367,6 @@ bucket_setstate(Bucket *self, PyObject *state)
     return Py_None;
 }
 
-static PyObject *
-bucket_has_key(Bucket *self, PyObject *key)
-{
-    return _bucket_get(self, key, 1);
-}
 
 static PyObject *
 bucket_setdefault(Bucket *self, PyObject *args)
@@ -1448,9 +1463,27 @@ bucket_contains(Bucket *self, PyObject *key)
         result = INT_AS_LONG(asobj) ? 1 : 0;
         Py_DECREF(asobj);
     }
+    else if (PyErr_ExceptionMatches(PyExc_KeyError)) {
+        PyErr_Clear();
+        result = 0;
+    }
     return result;
 }
 
+static PyObject *
+bucket_has_key(Bucket *self, PyObject *key)
+{
+    int result = -1;
+    result = bucket_contains(self, key);
+    if (result == -1) {
+        return NULL;
+    }
+
+    if (result)
+        Py_RETURN_TRUE;
+    Py_RETURN_FALSE;
+}
+
 /*
 ** bucket_getm
 **
@@ -1465,6 +1498,10 @@ bucket_getm(Bucket *self, PyObject *args)
     r = _bucket_get(self, key, 0);
     if (r)
         return r;
+    if (PyErr_ExceptionMatches(PyExc_TypeError)) {
+        PyErr_Clear();
+        PyErr_SetObject(PyExc_KeyError, key);
+    }
     if (!PyErr_ExceptionMatches(PyExc_KeyError))
         return NULL;
     PyErr_Clear();
